@@ -19,11 +19,11 @@
 */
 
 /*==============================================================================
-  
+
   $Id$
-  
+
   Driver for the Linux sam9407 driver
-  
+
 ==============================================================================*/
 
 /*
@@ -100,7 +100,7 @@ static Voice voices[SAM_NUM_VOICES];
 static int card=0;
 static int modfd=-1;
 
-static void commandLine(CHAR *cmdline)
+static void commandLine(const CHAR *cmdline)
 {
 	CHAR *ptr;
 
@@ -139,7 +139,7 @@ static SWORD sampleLoad(SAMPLOAD *s, int type)
 	SamModSamples modSamples;
 	SWORD handle;
 	SWORD *samples;
-int rc;
+	int rc;
 
 	for(handle=0; handle<SAM_NUM_BANKS; handle++)
 		if(!banks[handle].inUse)
@@ -182,7 +182,7 @@ int rc;
 	}
 
 	modSamples.handle=handle;
-	modSamples.data=samples;
+	modSamples.data=(unsigned short *)samples;
 	modSamples.size=bankP->length;
 	if((rc=ioctl(modfd, SAM_IOC_MOD_SAMPLES_LOAD, &modSamples))<0) {
 		MikMod_free(samples);
@@ -216,7 +216,7 @@ static ULONG realSampleLength(int type, SAMPLE *s)
 	return s->length<<1;
 }
 
-static BOOL init(void)
+static int init(void)
 {
 	int i;
 	Voice *voiceP;
@@ -255,7 +255,7 @@ static void exitHook(void)
 	}
 }
 
-static BOOL reset(void)
+static int reset(void)
 {
 	if(ioctl(modfd, SAM_IOC_MOD_RESET)<0)
 		return 1;
@@ -263,12 +263,12 @@ static BOOL reset(void)
 	return 0;
 }
 
-static BOOL setNumVoices(void)
+static int setNumVoices(void)
 {
 	return 0;
 }
 
-static BOOL playStart(void)
+static int playStart(void)
 {
 	ioctl(modfd, SAM_IOC_MOD_TIMER_START);
 	return 0;
@@ -457,10 +457,7 @@ static void voiceStop(UBYTE voice)
 
 static BOOL voiceStopped(UBYTE voice)
 {
-	if(voice>=SAM_NUM_VOICES)
-		return 1;
-
-	return voice<SAM_NUM_VOICES ? !voices[voice].playing : 1;
+	return (voice<SAM_NUM_VOICES)? !voices[voice].playing : 1;
 }
 
 static SLONG voiceGetPosition(UBYTE voice)
@@ -479,7 +476,7 @@ MIKMODAPI MDRIVER drv_sam9407={
 	"Linux sam9407 driver v1.0",
 	SAM_NUM_VOICES, 0,
 	"sam9407",
-
+	NULL,
 	commandLine,
 	isPresent,
 	sampleLoad,
